package edu.tum.motras.algorithms;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;

import edu.tum.motras.algorithms.util.Util;
import edu.tum.xmts.State;
import edu.tum.xmts.Transition;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;
import edu.tum.xmts.formula.Formula;

public class CompositionDMTS extends AbstractComposition {

	public CompositionDMTS(String string) {
		super(string);
	}

	public CompositionDMTS() {
		super();
	}

	@Override
	public Set<Type> getSupportedTypes() {
		return ImmutableSet.of(Type.MTS, Type.DMTS);
	}

	@Override
	public String toString() {
		return "Composition (DMTS)";
	}

	@Override
	protected void setObligation(State state, State leftState, State rightState,
			Map<Transition, Transition> translationMap) {
		Set<List<Transition>> leftCartesian = Sets.cartesianProduct(new LinkedList<>(Util
				.getDisjunctiveMustTransitions(leftState)));
		Set<List<Transition>> rightCartesian = Sets.cartesianProduct(new LinkedList<>(Util
				.getDisjunctiveMustTransitions(rightState)));
		Set<Set<Transition>> composedTran = new HashSet<>();

		for (Collection<Transition> transitions1 : leftCartesian) {
			for (Collection<Transition> transitions2 : rightCartesian) {
				Set<Transition> composedTransitions = new HashSet<>();

				for (Transition transition1 : transitions1) {
					if (translationMap.get(transition1) == null) {
						continue;
					}

					composedTransitions.add(translationMap.get(transition1));
				}

				for (Transition transition2 : transitions2) {
					if (translationMap.get(transition2) == null) {
						continue;
					}

					composedTransitions.add(translationMap.get(transition2));
				}

				if (!composedTransitions.isEmpty()) {
					composedTran.add(composedTransitions);
				}
			}
		}

		List<Set<Transition>> composedTranSet = new ArrayList<>(composedTran);

		if (composedTran.isEmpty()) {
			state.setObligation(Formula.TRUE);
		} else {
			state.setDisjunctiveMustObligation(Sets.cartesianProduct(composedTranSet));
		}
	}

	@Override
	protected void setup(TransitionSystem system) {
		system.setId("composition");
		system.setType(Type.DMTS);
	}

}
