package edu.tum.motras.algorithms;

import static edu.tum.motras.algorithms.util.Util.checkArguments;

import java.util.Set;

import com.google.common.collect.ImmutableSet;

import edu.tum.motras.algorithms.util.StatePair;
import edu.tum.xmts.State;
import edu.tum.xmts.TransitionSystem.Type;

public class Consistency implements Algorithm<Boolean> {

	@Override
	public Boolean apply(State... input) throws IllegalArgumentException {
		checkArguments(this, input);
		ConsistencyRelation relation = new ConsistencyRelation();
		return new Boolean(relation.apply(input).contains(new StatePair(input[0], input[1])));
	}

	@Override
	public int getArity() {
		return 2;
	}

	@Override
	public Set<Type> getSupportedTypes() {
		return ImmutableSet.of(Type.MTS, Type.DMTS);
	}

	@Override
	public String toString() {
		return "Consistency";
	}

	@Override
	public boolean isConfigurable() {
		return false;
	}

	@Override
	public void setConfiguration(String conf) {
		throw new UnsupportedOperationException("This algorithm cannot be configured");
	}
}
