package edu.tum.motras.algorithms;

import static edu.tum.motras.algorithms.util.Util.checkArguments;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import com.google.common.collect.Collections2;
import com.google.common.collect.ImmutableSet;

import edu.tum.motras.algorithms.util.StateSet;
import edu.tum.motras.algorithms.util.Util;
import edu.tum.xmts.Action;
import edu.tum.xmts.State;
import edu.tum.xmts.Transition;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;

public class DeterministicHullMTS extends WaitingQueueAlgorithm<StateSet> implements Algorithm<State> {

	@Override
	public State apply(State... input) {
		checkArguments(this, input);
		run(new StateSet(Collections.singleton(input[0])));
		return transitionSystem.getInitialState();
	}

	@Override
	public int getArity() {
		return 1;
	}

	@Override
	public Set<Type> getSupportedTypes() {
		return ImmutableSet.of(Type.MTS);
	}

	@Override
	public String toString() {
		return "Deterministic Hull (MTS)";
	}

	private State getState(Set<State> states) {
		StateSet set = new StateSet(states);
		queue(set);
		return stateMapper.map(set);
	}

	@Override
	protected void process(StateSet element) {
		State state = stateMapper.map(element);

		Set<Transition> mustTransitions = new HashSet<>();

		for (Action action : element.getActions()) {
			Set<State> successors = element.getSuccessors(action);

			if (successors.isEmpty()) {
				continue;
			}

			State nextState = getState(successors);
			Transition transition = state.createTransition(actionMapper.map(action), nextState);

			boolean mustTransition = true;

			for (State orginalState : element.getStates()) {
				Util.TransitionFilter transitionFilter = new Util.TransitionFilter(Collections.singleton(orginalState),
						Collections.singleton(action), successors);
				Collection<Transition> filteredTransitions = Collections2.filter(Util.getMustTransitions(orginalState),
						transitionFilter);

				if (filteredTransitions.isEmpty()) {
					mustTransition = false;
					break;
				}
			}

			if (mustTransition) {
				mustTransitions.add(transition);
			}
		}

		state.setMustObligation(mustTransitions);
	}

	@Override
	protected void setup(TransitionSystem system) {
		system.setId("deterministicHull");
	}

	@Override
	public boolean isConfigurable() {
		return false;
	}

	@Override
	public void setConfiguration(String conf) {
		throw new UnsupportedOperationException("This algorithm cannot be configured");
	}
}
