package edu.tum.motras.algorithms;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashSet;

import edu.tum.motras.algorithms.util.ActionMapper;
import edu.tum.motras.algorithms.util.StateMapper;
import edu.tum.motras.algorithms.util.Util;
import edu.tum.xmts.TransitionSystem;

public abstract class WaitingQueueAlgorithm<T> {

	protected TransitionSystem transitionSystem;
	protected ActionMapper actionMapper;
	protected StateMapper<T> stateMapper;
	protected LinkedHashSet<T> processed;
	protected LinkedHashSet<T> unprocessed;

	protected abstract void process(T element);

	protected void queue(T element) {
		if (!processed.contains(element)) {
			unprocessed.add(element);
		}
	}

	protected void run(T firstElement) {
		run(firstElement, Collections.singleton(firstElement));
	}

	protected void run(T firstElement, Collection<T> elements) {
		transitionSystem = new TransitionSystem("system");
		stateMapper = new StateMapper<>(transitionSystem);
		actionMapper = new ActionMapper(transitionSystem);
		processed = new LinkedHashSet<>();
		unprocessed = new LinkedHashSet<>();

		setup(transitionSystem);
		transitionSystem.setInitialState(stateMapper.map(firstElement));

		unprocessed.add(firstElement);
		unprocessed.addAll(elements);

		while (!unprocessed.isEmpty()) {
			T nextElement = Util.removeFirst(unprocessed);
			processed.add(nextElement);
			process(nextElement);
		}

		processed = null;
		unprocessed = null;
		actionMapper = null;
		stateMapper = null;
	}

	protected abstract void setup(TransitionSystem system);
}