package edu.tum.motras.algorithms.util;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Collection;
import java.util.Iterator;

import edu.tum.xmts.State;
import edu.tum.xmts.Transition;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.utils.TransitionSystemWriter;
import edu.tum.xmts.utils.XMTSReader;
import edu.tum.xmts.utils.XMTSReader.MalformedInputException;

public class MTSAWriter implements TransitionSystemWriter {

	public static void main(String[] args) throws MalformedInputException {
		String xmts1 = "mts test1[action a action b action c " + "state s1 [obligation all  a->s2]"
				+ "state s2 [obligation true b->s1 c->s1]]";

		XMTSReader reader = new XMTSReader();
		MTSAWriter writer = new MTSAWriter();
		System.out.print(writer.write(reader.read(xmts1)));
	}

	@Override
	public String write(TransitionSystem transitionSystem) {
		StringWriter stringWriter = new StringWriter();

		try {
			write(transitionSystem, stringWriter);
		} catch (IOException e) {
			e.printStackTrace();
		}

		return stringWriter.toString();
	}

	@Override
	public void write(TransitionSystem transitionSystem, Writer writer) throws IOException {
		Iterator<State> iterator = transitionSystem.states.getElements().iterator();

		while (iterator.hasNext()) {
			State state = iterator.next();

			write(state, writer);

			if (iterator.hasNext()) {
				writer.write(",\r\n");
			} else {
				writer.write(".");
			}
		}

		writer.flush();
	}

	private static void write(State state, Writer writer) throws IOException {
		writer.write("P" + state.getContext().getId() + "_" + state.getId());
		writer.write("=(");

		Iterator<Transition> iterator = state.getTransitions().iterator();
		Collection<Transition> mustTransitions = Util.getMustTransitions(state);

		while (iterator.hasNext()) {
			Transition transition = iterator.next();

			writer.write(transition.getAction().getId());

			if (!mustTransitions.contains(transition)) {
				writer.write("?");
			}

			writer.write("->");
			writer.write("P" + state.getContext().getId() + "_" + transition.getState().getId());

			if (iterator.hasNext()) {
				writer.write("|");
			}
		}

		writer.write(")");
	}
}