package edu.tum.motras.algorithms.util;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import edu.tum.xmts.State;
import edu.tum.xmts.Transition;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.utils.TransitionSystemWriter;
import edu.tum.xmts.utils.XMTSReader;
import edu.tum.xmts.utils.XMTSReader.MalformedInputException;

/**
 * Class to create a text output of an MTS, so that it is compatible with
 * <code>http://anna.fi.muni.cz/~xbenes3/MoTraS/</code>.
 * 
 * @author Salomon Sickert
 * 
 */
public class MoTraSWriter implements TransitionSystemWriter {

	public static void main(String[] args) throws MalformedInputException {
		String xmts1 = "mts test1[action a action b action c " + "state s1 [obligation all  a->s2]"
				+ "state s2 [obligation true b->s1 c->s1]]";

		XMTSReader reader = new XMTSReader();
		MoTraSWriter writer = new MoTraSWriter();
		System.out.print(writer.write(reader.read(xmts1)));
	}

	@Override
	public String write(TransitionSystem transitionSystem) {
		StringWriter stringWriter = new StringWriter();

		try {
			write(transitionSystem, stringWriter);
		} catch (IOException e) {
			e.printStackTrace();
		}

		return stringWriter.toString();
	}

	@Override
	public void write(TransitionSystem transitionSystem, Writer writer) throws IOException {
		BufferedWriter bufferedWriter = new BufferedWriter(writer);

		for (State state : transitionSystem.states.getElements()) {
			write(state, bufferedWriter);
			bufferedWriter.write("\n");
		}

		bufferedWriter.flush();
	}

	private static void write(State state, Writer writer) throws IOException {
		writer.write(state.getId().toUpperCase() + " = ");

		Iterator<Set<Transition>> iterator = Util.getDisjunctiveMustTransitions(state).iterator();
		Iterator<Transition> mayIterator;
		Set<Transition> mayTransitions = new HashSet<>(state.getTransitions());

		while (iterator.hasNext()) {
			Set<Transition> transitions = iterator.next();

			if (transitions.size() > 1) {
				writer.write("{");
			}

			Iterator<Transition> transitionIterator = transitions.iterator();

			while (transitionIterator.hasNext()) {
				Transition transition = transitionIterator.next();

				write(transition, writer);
				mayTransitions.remove(transition);

				if (transitionIterator.hasNext()) {
					writer.write(" | ");
				}
			}

			if (transitions.size() > 1) {
				writer.write("}");
			}

			if (transitionIterator.hasNext() || !mayTransitions.isEmpty()) {
				writer.write(" + ");
			}
		}

		mayIterator = mayTransitions.iterator();

		while (mayIterator.hasNext()) {
			writer.write("?");
			write(mayIterator.next(), writer);

			if (mayIterator.hasNext()) {
				writer.write("+");
			}
		}
	}

	private static void write(Transition transition, Writer writer) throws IOException {
		writer.write(transition.getAction().getId().toLowerCase() + ".");
		writer.write(transition.getState().getId().toUpperCase());
	}
}