package edu.tum.motras.legacy.pmtsrc.util.formula;

import it.unimi.dsi.fastutil.ints.Int2IntMap;
import it.unimi.dsi.fastutil.ints.Int2IntOpenHashMap;

import java.util.HashMap;
import java.util.Map;

import edu.tum.motras.algorithms.util.StatePair;
import edu.tum.xmts.Transition;

public class PrefixedTransitionFactory extends AbstractLiteralFactory {
	final private Map<StatePair, Int2IntMap> prefixedTransitionMap;
	final private LiteralFactory<Transition> library;

	protected static AbstractLiteralFactory lookup(int variable) {
		for (AbstractLiteralFactory factory : initializedFactories) {
			LiteralRange range = factory.getLiteralRange();

			if (range.lowerBound <= variable && variable < range.upperBound) {
				return factory;
			}
		}

		return null;
	}

	public PrefixedTransitionFactory(LiteralFactory<Transition> l) {
		super(Domain.PREFIXEDTRANSITION);
		library = l;
		prefixedTransitionMap = new HashMap<>();
	}

	protected Literal getLiteral(StatePair prefix, Literal transition) {
		Int2IntMap m = prefixedTransitionMap.get(prefix);

		if (m == null) {
			prefixedTransitionMap.put(prefix, m = new Int2IntOpenHashMap());
		}

		return new Literal(getVariable(m, transition.variable), transition.negated);
	}

	public Literal getLiteral(StatePair prefix, Transition transition) {
		return getLiteral(prefix, library.getLiteral(transition));
	}

	public TreeBoolean prefix(StatePair prefix, TreeBoolean formula) {
		return formula.accept(new TransitionPrefixer(prefix));
	}

	private class TransitionPrefixer implements TreeBoolean.Visitor<TreeBoolean> {

		final private StatePair prefix;

		TransitionPrefixer(StatePair p) {
			prefix = p;
		}

		@Override
		public TreeBoolean visit(Composition c) {
			TreeBoolean[] children = c.children;

			for (int i = 0; i < children.length; i++) {
				children[i] = children[i].accept(this);
			}

			return c;
		}

		@Override
		public TreeBoolean visit(Literal l) {
			if (l.isTrue() || l.isFalse()) {
				return l;
			}

			if (lookup(l.variable).domain.equals(Domain.TRANSITION)) {
				return getLiteral(prefix, l);
			}

			return l;
		}
	}
}
