package edu.tum.xmts;

import static com.google.common.base.Preconditions.checkState;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

import edu.tum.xmts.formula.Formula;

@NonNullByDefault
public class Action extends GenericRecord<TransitionSystem> {

	/* XMTS Properties */

	public static final String PROP_REQUIREMENT = "requirement";
	public static final String PROP_RUNNING_COST = "running_cost";

	private @Nullable Formula requirement;
	private @Nullable Integer runningCost;

	Action(TransitionSystem context, String id) {
		super(context, id);
	}

	public void clearRequirement() {
		setRequirement(null);
	}

	public @Nullable Formula getRequirement() {
		return requirement;
	}

	public void setRequirement(@Nullable Formula requirement) {
		checkState(isSupportedProperty(PROP_REQUIREMENT));

		Formula oldRequirement = this.requirement;
		this.requirement = requirement;
		propertyChangeSupport.firePropertyChange(PROP_REQUIREMENT, oldRequirement, requirement);
	}

	public void clearRunningCost() {
		setRunningCost(null);
	}

	public @Nullable Integer getRunningCost() {
		return runningCost;
	}

	public void setRunningCost(@Nullable Integer runningCost) {
		checkState(isSupportedProperty(PROP_RUNNING_COST));

		Integer oldRunningCost = this.runningCost;
		this.runningCost = runningCost;
		propertyChangeSupport.firePropertyChange(PROP_REQUIREMENT, oldRunningCost, runningCost);
	}

	@Override
	public boolean isSupportedProperty(String propName) {
		if (super.isSupportedProperty(propName)) {
			return true;
		}

		// Only mtsd+dps allows additional properties for actions and parameters
		if (getContext().getType() != TransitionSystem.Type.MTSD_DPS) {
			return false;
		}

		return propName.equals(PROP_RUNNING_COST) || propName.equals(PROP_REQUIREMENT);
	}

}
