package edu.tum.xmts.formula;

import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.base.Preconditions.checkState;

import java.util.Collection;

import edu.tum.xmts.Parameter;
import edu.tum.xmts.State;
import edu.tum.xmts.Transition;
import edu.tum.xmts.formula.Formula.Atom;
import edu.tum.xmts.formula.Formula.Conjunction;
import edu.tum.xmts.formula.Formula.Disjunction;
import edu.tum.xmts.formula.Formula.Equivalence;
import edu.tum.xmts.formula.Formula.Implication;
import edu.tum.xmts.formula.Formula.Negation;

public class CheckContextVisitor implements VoidVisitor {

	private final State context;

	public CheckContextVisitor(State context) {
		this.context = context;
	}

	@Override
	public void visit(Atom<?> atom) {
		Object payload = atom.getPayload();

		checkNotNull(payload);

		if (payload instanceof Transition) {
			Transition transition = (Transition) atom.getPayload();
			checkState(transition.getContext() == context);
			checkState(context.getTransitions().contains(transition));
		} else {
			checkState(payload instanceof Parameter || payload instanceof Boolean);
		}
	}

	@Override
	public void visit(Conjunction conjunction) {
		visit(conjunction.children);
	}

	@Override
	public void visit(Disjunction disjunction) {
		visit(disjunction.children);
	}

	@Override
	public void visit(Equivalence equivalence) {
		visit(equivalence.children);
	}

	@Override
	public void visit(Implication implication) {
		visit(implication.children);
	}

	@Override
	public void visit(Negation negation) {
		visit(negation.children);
	}

	private void visit(Collection<Formula> formulas) {
		for (Formula formula : formulas) {
			formula.accept(this);
		}
	}
}
