package edu.tum.xmts.formula;

import java.util.ArrayList;
import java.util.List;

import edu.tum.xmts.formula.Formula.Atom;
import edu.tum.xmts.formula.Formula.Conjunction;
import edu.tum.xmts.formula.Formula.Disjunction;
import edu.tum.xmts.formula.Formula.Equivalence;
import edu.tum.xmts.formula.Formula.Implication;
import edu.tum.xmts.formula.Formula.Negation;

public class RemoveVisitor implements Visitor<Formula> {

	private final Object object;

	public RemoveVisitor(Object object) {
		this.object = object;
	}

	@Override
	public Formula visit(Atom<?> atom) {
		Object payload = atom.getPayload();

		if (payload == object) {
			return Formula.TRUE;
		}

		return atom;
	}

	private List<Formula> visit(List<Formula> formulas) {
		List<Formula> list = new ArrayList<>(formulas.size());

		for (Formula formula : formulas) {
			list.add(formula.accept(this));
		}

		return list;
	}

	@Override
	public Formula visit(Conjunction conjunction) {
		return Formula.and(visit(conjunction.children));
	}

	@Override
	public Formula visit(Disjunction disjunction) {
		return Formula.or(visit(disjunction.children));
	}

	@Override
	public Formula visit(Equivalence equivalence) {
		return new Equivalence(equivalence.children.get(0).accept(this), equivalence.children.get(1).accept(this));
	}

	@Override
	public Formula visit(Implication implication) {
		return new Implication(implication.children.get(0).accept(this), implication.children.get(1).accept(this));
	}

	@Override
	public Formula visit(Negation negation) {
		return new Negation(negation.children.get(0).accept(this));
	}

}