package edu.tum.motras.algorithms.junit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.util.LinkedList;
import java.util.Set;

import edu.tum.motras.algorithms.Algorithm;
import edu.tum.xmts.State;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;
import edu.tum.xmts.utils.XMTSGenerator;
import edu.tum.xmts.utils.XMTSGenerator.XMTSGeneratorConfiguration;

public class AlgorithmTestUtil {

	public static LinkedList<TransitionSystem> generate(Type type) {
		return generate(type, false);
	}

	public static LinkedList<TransitionSystem> generate(Type type, boolean deterministic) {
		LinkedList<TransitionSystem> systems = new LinkedList<>();
		XMTSGenerator generator = new XMTSGenerator();
		XMTSGeneratorConfiguration configuration = new XMTSGeneratorConfiguration();

		configuration.type = type;
		configuration.parameters = 0;
		configuration.actions = 3;

		configuration.clusterInterfaceNodes = 1;
		configuration.clusterNodeDegree = 0;
		configuration.clusterSize = 1;

		configuration.systemNodeDegree = deterministic ? 3 : 4;
		configuration.systemSize = 10;

		configuration.deterministic = deterministic;

		if (type == Type.PMTS) {
			configuration.parameters = 2;
		}

		for (int i = 0; i < 20; i++) {
			systems.add(generator.generate(configuration, 13l + i));
		}

		return systems;
	}

	public static void testGetArity(Algorithm<?> algorithm, int arity) {
		assertEquals(arity, algorithm.getArity());

		for (int i = 0; i < 10; i++) {
			if (i == algorithm.getArity()) {
				continue;
			}

			try {
				algorithm.apply(new State[i]);
			} catch (IllegalArgumentException exp) {
				continue;
			} catch (Exception exp) {
				fail("Wrong exception");
			}

			fail("Missing IllegalArgumentException");
		}
	}

	public static void testGetSupportedTypes(Algorithm<?> algorithm, Set<Type> supportedTypes) {
		assertEquals(supportedTypes, algorithm.getSupportedTypes());

		for (Type type : TransitionSystem.Type.values()) {
			if (algorithm.getSupportedTypes().contains(type)) {
				continue;
			}

			TransitionSystem[] transitionSystems = new TransitionSystem[algorithm.getArity()];
			State[] states = new State[algorithm.getArity()];

			for (int i = 0; i < algorithm.getArity(); i++) {
				transitionSystems[i] = new TransitionSystem("mts" + i);
				transitionSystems[i].setType(type);
				states[i] = transitionSystems[i].states.create();
			}

			try {
				algorithm.apply(states);
			} catch (IllegalArgumentException exp) {
				continue;
			} catch (Exception exp) {
				fail("Wrong exception");
			}

			fail("Missing IllegalArgumentException");
		}
	}
}