package edu.tum.motras.algorithms.junit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.Collection;

import org.junit.Before;
import org.junit.Test;

import com.google.common.collect.ImmutableSet;

import edu.tum.motras.algorithms.Algorithm;
import edu.tum.motras.algorithms.MaxImplementation;
import edu.tum.motras.algorithms.ModalRefinementQSat;
import edu.tum.motras.algorithms.util.Util;
import edu.tum.xmts.State;
import edu.tum.xmts.Transition;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;
import edu.tum.xmts.utils.XMTSReader;

public class MaxImplementationTest {

	private MaxImplementation maxImplementation;
	private Algorithm<Boolean> modalRefinement;

	@Before
	public void setUp() throws Exception {
		maxImplementation = new MaxImplementation();
		modalRefinement = new ModalRefinementQSat();
	}

	@Test
	public void testApply() throws Exception {
		String xmts = "mts test1[action a action b " + "state s1 [obligation a.s3 b->s2 a->s3]" + "state s2 "
				+ "state s3 [obligation true b->s3]]";

		XMTSReader reader = new XMTSReader();
		TransitionSystem system1 = reader.read(xmts);

		Collection<Transition> aTran;
		Collection<Transition> bTran;
		Collection<Transition> mustTran;

		State s1 = maxImplementation.apply(system1.states.lookup("s1"));
		assertTrue(modalRefinement.apply(s1, system1.states.lookup("s1")));

		/* Test TransitionSystem */
		TransitionSystem compositionSystem = s1.getContext();
		assertEquals(Type.MTS, compositionSystem.getType());
		assertEquals(compositionSystem.parameters.getElements().toString(), 0, compositionSystem.parameters
				.getElements().size());
		assertEquals(compositionSystem.actions.getElements().toString(), 2, compositionSystem.actions.getElements()
				.size());
		assertEquals(compositionSystem.states.getElements().toString(), 3, compositionSystem.states.getElements()
				.size());

		/* Test State s1 */
		aTran = s1.getTransitions("a");
		bTran = s1.getTransitions("b");

		assertEquals(aTran.toString(), 1, aTran.size());
		assertEquals(bTran.toString(), 1, bTran.size());

		mustTran = Util.getMustTransitions(s1);

		State s2 = bTran.iterator().next().getState();
		State s3 = aTran.iterator().next().getState();

		assertTrue(mustTran.contains(aTran.iterator().next()));
		assertTrue(mustTran.contains(bTran.iterator().next()));

		/* Test State s2 */
		aTran = s2.getTransitions("a");
		bTran = s2.getTransitions("b");

		assertEquals(aTran.toString(), 0, aTran.size());
		assertEquals(bTran.toString(), 0, bTran.size());

		mustTran = Util.getMustTransitions(s2);

		assertTrue(mustTran.isEmpty());

		/* Test State s3 */
		aTran = s3.getTransitions("a");
		bTran = s3.getTransitions("b");

		assertEquals(aTran.toString(), 0, aTran.size());
		assertEquals(bTran.toString(), 1, bTran.size());

		mustTran = Util.getMustTransitions(s3);

		assertEquals(1, mustTran.size());
		assertTrue(mustTran.contains(bTran.iterator().next()));
	}

	@Test
	public void testApplyRandom() throws Exception {
		for (TransitionSystem transitionSystem : AlgorithmTestUtil.generate(Type.DMTS)) {
			State s1 = transitionSystem.states.getElements().get(0);
			State s1MaxImplementation = maxImplementation.apply(s1);
			assertTrue(modalRefinement.apply(s1MaxImplementation, s1));
		}
	}

	@Test
	public void testGetArity() {
		AlgorithmTestUtil.testGetArity(maxImplementation, 1);
	}

	@Test
	public void testGetSupportedTypes() {
		AlgorithmTestUtil.testGetSupportedTypes(maxImplementation, ImmutableSet.of(Type.MTS, Type.DMTS));
	}

}
