package edu.tum.motras.algorithms.junit;

import static org.junit.Assert.assertTrue;

import java.util.LinkedList;

import org.junit.Before;
import org.junit.Test;

import com.google.common.collect.ImmutableSet;

import edu.tum.motras.algorithms.Algorithm;
import edu.tum.motras.algorithms.CompositionMTS;
import edu.tum.motras.algorithms.MaxImplementation;
import edu.tum.motras.algorithms.ModalRefinementFixedPoint;
import edu.tum.motras.algorithms.Quotient;
import edu.tum.xmts.State;
import edu.tum.xmts.TransitionSystem;
import edu.tum.xmts.TransitionSystem.Type;
import edu.tum.xmts.utils.XMTSGenerator;
import edu.tum.xmts.utils.XMTSGenerator.XMTSGeneratorConfiguration;

public class QuotientTest {

	private Quotient quotient;

	@Before
	public void setUp() throws Exception {
		quotient = new Quotient();
	}

	public static LinkedList<TransitionSystem> generate(Type type, boolean deterministic) {
		LinkedList<TransitionSystem> systems = new LinkedList<>();
		XMTSGenerator generator = new XMTSGenerator();
		XMTSGeneratorConfiguration configuration = new XMTSGeneratorConfiguration();

		configuration.type = type;
		configuration.parameters = 0;
		configuration.actions = 2;

		configuration.clusterInterfaceNodes = 1;
		configuration.clusterNodeDegree = 0;
		configuration.clusterSize = 1;

		configuration.systemNodeDegree = 2;
		configuration.systemSize = 4;

		configuration.deterministic = deterministic;

		for (int i = 0; i < 4; i++) {
			systems.add(generator.generate(configuration, 13l + i));
		}

		return systems;
	}

	@Test
	public void testApplyRandom() throws Exception {
		LinkedList<TransitionSystem> systems = generate(Type.MTS, true);
		CompositionMTS compositionMTS = new CompositionMTS("a0, a1, a2");
		MaxImplementation maxImplementation = new MaxImplementation();
		Algorithm<Boolean> modalRefinement = new ModalRefinementFixedPoint();

		/*
		 * X <= A/B iff. X||B <= A and A/B || B <= A
		 */
		while (systems.size() > 1) {
			State A = systems.remove().getInitialState();
			State B = systems.remove().getInitialState();
			State A_B = quotient.apply(A, B);

			if (A_B == null) {
				continue;
			}

			State X = maxImplementation.apply(A_B);
			State X_B = compositionMTS.apply(X, B);
			State A_B_B = compositionMTS.apply(A_B, B);

			assertTrue(modalRefinement.apply(X_B, A));
			assertTrue(modalRefinement.apply(A_B_B, A));
		}
	}

	@Test
	public void testGetArity() {
		AlgorithmTestUtil.testGetArity(quotient, 2);
	}

	@Test
	public void testGetSupportedTypes() {
		AlgorithmTestUtil.testGetSupportedTypes(quotient, ImmutableSet.of(Type.MTS));
	}

}
